@@ -21,6 +21,8 @@
  * december 1997
  */
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <rpc/rpc.h>
 #include <sys/time.h>
 #ifdef irix
@@ -48,7 +50,9 @@
 #endif
 #include "nfsproto.h"
 #include "admproto.h"
+#include "cattach.h"
 #include "cfs.h"
+#include "getpass.h"
 
 #ifdef SOLARIS2X
 /* NOTE!  delete the #define statfs below if this won't compile on
@@ -64,17 +68,15 @@
 #define IDLE 0
 #endif
 
-main(argc,argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char *argv[])
 {
 	cfs_attachargs ap;
 	char *pw;
-	char pword[256];
+	char pword[MAX_PASSPHRASE_LEN+1];
 	int n;
 	int status;
 	cfsstat ret;
-	char *getpassword();
 	char dir[1024]; char buf[1024];
 	char ins[1024];
 #ifdef	ultrix
@@ -91,6 +93,9 @@ main(argc,argv)
 	char kname[1024]; /* indirect key file */
 	char sname[1024];
 	char lname[1024]; /* ..data name */
+	char vname[1024]; /* datastore version file */
+	long ds, dsversion;
+	long *dsp;
 	int smsize;
 	int cfmt=0;
 	static struct timeval tout = {60,0};
@@ -100,6 +105,7 @@ main(argc,argv)
 	char *dirarg=NULL;
 	char *namearg=NULL;
 	int keycheck=1;
+	int l;
 
 	ap.highsec=1;
 	while (--argc) if (**++argv == '-') {
@@ -159,18 +165,38 @@ main(argc,argv)
 		fprintf(stderr,"usage: cattach [-l] [-t timeout] [-i idle] dir name\n");
 		exit(1);
 	}
-	if (*dirarg!='/') {
-		if (getcwd(buf,1024) == NULL) {
+	if (*dirarg != '/') {
+		if (getcwd(buf, 1024) == NULL) {
 			fprintf(stderr,"Can't stat current directory\n");
 			exit(1);
 		}
-		sprintf(dir,"%s/%s",buf,dirarg);
+		l = snprintf(dir, 1024, "%s/%s",buf,dirarg);
+		if (l < 0 || l >= 1024) {
+			fprintf(stderr,"Directory name too long\n");
+			exit(1);
+		}
 	} else
-		strcpy(dir,dirarg);
-	sprintf(lname,"%s/..data",dir,1024);
-	sprintf(kname,"%s/..k",dir,1024);
+		l = snprintf(dir, 1024, "%s", dirarg);
+		if (l < 0 || l >= 1024) {
+			fprintf(stderr, "Directory name too long: %s\n", dirarg);
+			exit(1);
+		}
+	l = snprintf(lname, 1024, "%s/..data", dir);
+	if (l < 0 || l >= 1024) {
+		fprintf(stderr, "data name too long: %s\n", lname);
+		exit(1);
+	}
+	sprintf(kname, "%s/..k", dir); /* Checked for length above */
+	sprintf(vname, "%s/..v", dir); /* Checked for length above */
+	if ((fp = fopen(vname, "r")) != NULL) {
+		fscanf(fp, "%ld", &dsversion);
+		fclose(fp);
+	} else { /* Obviously created under an old version */
+		dsversion = 0;
+	}
 	if (chdir(lname) >= 0)
-		strcpy(dir,lname);
+		/* OK to strcpy here; dir and lname are the same length */
+		strcpy(dir, lname);
 	else if (chdir(dir)<0) {
 		perror(dirarg);
 		exit(1);
@@ -188,7 +214,11 @@ main(argc,argv)
 	}
 #endif
 	ap.dirname=dir;
-	strcpy(ins,namearg);
+	l = snprintf(ins, 1024, "%s", namearg);
+	if (l < 0 || l >= 1024) {
+		fprintf(stderr, "Name argument too long; %s\n", namearg);
+		exit(1);
+	};
 	*namearg='\0'; /* weak attempt to hide .instance in ps output */
 	ap.name=ins;
 	if (keycheck) {
@@ -197,29 +227,33 @@ main(argc,argv)
 			exit(1);
 		}
 	} else {
-		if (fgets(pword,256,stdin) == NULL) {
+		if (fgets(pword, MAX_PASSPHRASE_LEN, stdin) == NULL) {
 			perror("cmkdir");
 			exit(1);
 		}
 		pw=pword;
-		pw[255]='\0';
+		pw[MAX_PASSPHRASE_LEN]='\0';
 		n=strlen(pw);
 		if ((n>0) && (pw[n-1] == '\n'))
 			pw[n-1] = '\0';
 	}
- 	sprintf(cname,"%s/..c",dir);
-	sprintf(sname,"%s/..s",dir);
+	l = snprintf(cname, 1024, "%s/..c", dir);
+	if (l < 0 || l >= 1024) {
+		fprintf(stderr, "cname too long; %s\n", dir);
+		exit(1);
+	}
+	sprintf(sname, "%s/..s", dir); /* Checked for length above */
 	if ((fp=fopen(cname,"r")) == NULL) {
 		ciph=CFS_STD_DES;
 	} else {
-		fscanf(fp,"%d",&ciph);
+		fscanf(fp, "%d", &ciph);
 		fclose(fp);
 	}
 	if ((fp=fopen(kname,"r")) == NULL) {
 		cfmt=0;
 	} else {
 		cfmt=1;
-		if (fread(ekey,32,1,fp) < 0)
+		if (fread(ekey, 32, 1, fp) < 1)
 			cfmt=0;
 		fclose(fp);
 	}
@@ -260,8 +294,26 @@ main(argc,argv)
 		exit(1);
 	}
 	cln->cl_auth = authunix_create_default();
-	if ((status = clnt_call(cln,ADMPROC_ATTACH,xdr_cfs_attachargs,&ap,
-				xdr_cfsstat,&ret,tout)) != RPC_SUCCESS) {
+
+	/* First check version matches */
+	if ((dsp = admproc_getversion_2(&ds, cln)) == NULL) {
+		DEBUGMSG("No reply from RPC version, assuming dsversion 0\n");
+		ds = 0; /* Version is zero if the function doesn't exist */
+	} else
+		ds = *dsp;
+	DEBUGMSG("dsversion detected as %ld\n", ds);
+	if (ds != dsversion) {
+		fprintf(stderr, "Datastore version of directory is %ld, cfsd is %ld.  Rebuild required for data safety; please see README.\n", dsversion, ds);
+		exit(1);
+	}
+
+		/* XXX
+		 * Some casting in the next line-- should now be correct...
+		 * But weird bugs may happen if we're unlucky.
+		 */
+	status = clnt_call(cln, ADMPROC_ATTACH, (xdrproc_t)&xdr_cfs_attachargs,
+			   &ap, (xdrproc_t)&xdr_cfsstat, &ret, tout);
+	if (status != RPC_SUCCESS) {
 		clnt_perrno(status);
 		exit(1);
 	}
@@ -277,8 +329,8 @@ struct {
         int tail;
 } argq = {{0},0,0};
 
-enq(f)
-     char f;
+void
+enq(char f)
 {
 	argq.tail++;
 	argq.tail %= QS;
@@ -289,7 +341,8 @@ enq(f)
 	argq.data[argq.tail]=f;
 }
 
-deq()
+int
+deq(void)
 {
 	if (argq.head==argq.tail)
 		return -1;
