@@ -18,15 +18,17 @@
 /*
  * client side detach
  */
-#include <stdio.h>
 #include <rpc/rpc.h>
+
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
 
-main(argc,argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char *argv[])
 {
 	cfs_detachargs ap;
 	char *pw;
@@ -39,9 +41,9 @@ main(argc,argv)
 	}
 	ap.name=argv[1];
 	ap.uid=getuid();
-	if ((status = callrpc("localhost",ADM_PROGRAM,ADM_VERSION,
-			    ADMPROC_DETACH,xdr_cfs_detachargs,&ap,
-			    xdr_cfsstat,&ret)) !=0) {
+	if ((status = callrpc("localhost", ADM_PROGRAM, ADM_VERSION,
+			    ADMPROC_DETACH, (xdrproc_t)&xdr_cfs_detachargs,
+			    &ap, (xdrproc_t)&xdr_cfsstat, &ret)) !=0) {
 		clnt_perrno(status);
 		exit(1);
 	}
