@@ -21,47 +21,23 @@
  */
 
 #include <sys/types.h>
+#include <sys/errno.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#include <sys/errno.h>
-#ifdef NeXT
-#include <sys/dir.h>
-#define dirent direct
-#else
-#include <dirent.h>
-#endif
+
 #include <rpc/rpc.h>
+
+#include <dirent.h>
 #include <fcntl.h>
 #include <stdio.h>
-#ifdef hpux
-#define NO_UTIMES
-#include <time.h>
-#endif
-#ifdef	ultrix
-#include <time.h>
-#endif	/* ultrix */
-#ifdef NO_UTIMES
-#include <utime.h>
-#endif
+#include <stdlib.h>
+#include <string.h>
 
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
 
-#ifdef NO_UTIMES
-/* simulate utimes with utime */
-int utimes(s,tv)
-char *s;
-struct timeval tv[2];
-{
-    struct utimbuf ut;
-    ut.actime = tv[0].tv_sec;
-    ut.modtime = tv[1].tv_sec;
-    return utime(s,&ut);
-}
-#endif
-
-#define hash(x) ((u_long)((x)&HMASK))
+#define hash(x) ((uint32_t)((x)&HMASK))
 int inst = 0;/* starting point */
 instance *instances[NINSTANCES];
 
@@ -75,16 +51,10 @@ int cfserrno=NFS_OK;
  * get an encrypted block from fd, which damn well better be valid
  */
 int
-readblock(blk,fd,offset,len,key,vect)
-     char *blk;     /* data */
-     int fd;        /* fd */
-     u_long offset; /* offset from byte zero */
-     u_long len;    /* blk len */
-     cfskey *key;     /* des key */
-     char *vect;  /* perturbation vector */
+readblock(char *blk, int fd, u_long offset, u_long len, cfskey *key, char *vect)
 {
-	u_long begin;
-	u_long fronterr, totlen,datalen;
+	uint32_t begin;
+	uint32_t fronterr, totlen, datalen;
 	int iolen;
 	int readmore=0;
 	int userbytes;
@@ -121,21 +91,15 @@ readblock(blk,fd,offset,len,key,vect)
  * write an encrypted block to fd
  */
 int
-writeblock(blk,fd,offset,len,key,vect)
-     char *blk;     /* data */
-     int fd;        /* fd */
-     u_long offset; /* offset from byte zero */
-     u_long len;    /* blk len */
-     cfskey *key;     /* des key */
-     char *vect;    /* perturb. vector */
+writeblock(char *blk, int fd, u_long offset, u_long len,
+    cfskey *key, char *vect)
 {
-	u_long begin, end;
-	u_long fronterr, totlen,datalen;
-	int iolen;
+	uint32_t begin, end;
+	uint32_t fronterr, totlen, datalen;
+	uint32_t iolen;
 	int headlen;
 	int writemore=0;
 	struct stat sb;
-	u_long mask = ~0;	/*tells us whether to expand file */
 	char buf[8216];	/* big enough, may not even need it */
 
 	/* first, normalize to the proper boundries */
@@ -190,39 +154,38 @@ writeblock(blk,fd,offset,len,key,vect)
  *  leave "." and ".." unmolested
  */
 char *
-encryptname(key,s)
-     cfskey *key;
-     char *s;
+encryptname(cfskey *key, char *s)
 {
 	static char cryptname[NFS_MAXNAMLEN+1];
-	u_char cryptstring[MAXCLEARNAME+CFSBLOCK+1];
-	u_char x[3];
+	char cryptstring[MAXCLEARNAME+CFSBLOCK+1];
+	char x[3];
 	u_long l;
 	int i;
 
 	if ((s==NULL) || ((l=strlen(s)+1)>MAXCLEARNAME))
 		return NULL;
-	if (!strcmp(s,".") || !strcmp(s,".."))
-		return s;
+	if (is_dot_or_dotdot(s))
+		return (s);
 	l=(l+(CFSBLOCK-1)) & (~(CFSBLOCK-1));
-	bzero((char *)cryptstring,l);
-	strcpy(cryptstring,s);
-	chksum(cryptstring,l);
+	bzero(cryptstring, l);
+	strcpy(cryptstring, s); /* Cannot overflow; checked above */
+	chksum(cryptstring, l);
 	doencrypt(key,cryptstring,l,10241,zerovect);
 	cryptname[0]='\0';
 	for (i=0; i<l; i++) {
-		sprintf((char *)x,"%02x",cryptstring[i]);
-		strcat(cryptname,x);
+		sprintf(x,"%02x", (u_char)(cryptstring[i]));
+		cryptname[2*i] = x[0];
+		cryptname[2*i+1] = x[1];
 	}
+	cryptname[2*i] = '\0';
 	return cryptname;
 }
 
 /*
  * set high order bits
  */
-chksum(s,l)
-     char *s;
-     long l;
+void
+chksum(char *s, long l)
 {
 	u_long acc;
 	int i;
@@ -245,9 +208,7 @@ chksum(s,l)
  *  leaving "." and ".."
  */
 char *
-decryptname(key,s)
-     cfskey *key;
-     char *s;
+decryptname(cfskey *key, char *s)
 {
 	static char clearstring[MAXCLEARNAME+CFSBLOCK+1];
 	char x[3];
@@ -258,8 +219,8 @@ decryptname(key,s)
 
 	if (s==NULL)
 		return NULL;
-	if (!strcmp(s,".") || !strcmp(s,".."))
-		return s;
+	if (is_dot_or_dotdot(s))
+		return (s);
 	bzero((char *)clearstring,MAXCLEARNAME+CFSBLOCK+1);
 	/* unencode the string */
 	p=s;
@@ -285,12 +246,8 @@ decryptname(key,s)
 	return clearstring;
 }
 
-doencrypt(k,s,l,salt,vect)
-     cfskey *k;
-     char *s;
-     int l;
-     int salt;
-     char *vect;
+void
+doencrypt(cfskey *k, char *s, int l, uint32_t salt, char *vect)
 {
 	int i,j,smsize;
 	
@@ -301,26 +258,22 @@ doencrypt(k,s,l,salt,vect)
 				^ vect[j]
 				^ (((i+j+salt)/smsize)&0377);
 				/* makes big offsets have different masks */
-		cipher(k,&s[i],0);
+		cipher(k, (u_char *)&s[i], 0);
 		for (j=0; j<8; j++)
 			s[i+j] ^= k->secmask[(i+j+salt)%smsize];
 	}
 }
 
-dodecrypt(k,s,l,salt,vect)
-     cfskey *k;
-     char *s;
-     int l;
-     int salt;
-     char *vect;
+void
+dodecrypt(cfskey *k, char *s, int l, uint32_t salt, char *vect)
 {
-	int i,j,smsize;
+	int i, j, smsize;
 	
 	smsize=k->smsize;
 	for (i=0; i<l; i+=8) {
 		for (j=0; j<8; j++)
 			s[i+j] ^= k->secmask[(i+j+salt)%smsize];
-		cipher(k,&s[i],1);
+		cipher(k, (u_char *)&s[i], 1);
 		for (j=0; j<8; j++)
 			s[i+j] ^= k->primask[(i+j+salt)%smsize]
 				^ vect[j]
@@ -332,52 +285,53 @@ dodecrypt(k,s,l,salt,vect)
 /*
  * convert size of data to size of block to be written
  */
-vtod(d)
-     int d;
+int
+vtod(int d)
 {
-	return isbndry(d)?d:(d+CFSBLOCK);
+	return isbndry(d) ? d : (d+CFSBLOCK);
 }
 
 
 /*
  * convert file size into #of valid bits
  */
-dtov(d)
-     int d;
+int
+dtov(int d)
 {
-	return isbndry(d)?d:(d-CFSBLOCK);
+	return isbndry(d) ? d : (d-CFSBLOCK);
 }
 
 /*
  * is block size a CFS boundry?
  */
-isbndry(d)
-     int d;
+int
+isbndry(int d)
 {
-	return ((d&(CFSBLOCK-1))==0);
+	return ((d & (CFSBLOCK - 1)) == 0);
 }
 
 
-static fh_u roothandle;
+static nfs_fh roothandle;
+#ifdef SHORTLINKS
+cfs_fileid rootnode={1,0,"\0\0\0\0\0\0\0\0","/NOWHERE/null",NULL,NULL,0,0,0,NULL,NULL};/* fileid=1; should be unique */
+#else /* SHORTLINKS */
 cfs_fileid rootnode={1,0,"\0\0\0\0\0\0\0\0","/NOWHERE/null",NULL,NULL,0,NULL,NULL};/* fileid=1; should be unique */
+#endif /* SHORTLINKS */
 nfstime roottime={0,0};
 
 /* Had to change this - someone else was using the same magic number! */
-/* PLEASE change this value if you're going to re-use this code for
+/* PLEASE change this value if you're going to re-use this code for */
 /* something else!  Otherwise your FS and CFS can't both run... */
 /* static u_char magictest[8]={0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef}; */
 static u_char magictest[8]="M.Blaze";
 
 static cfs_fileid *
-findh(id,ins
+findh(int id, int ins
 #ifdef SHORTLINKS
-	    ,linkid)
-     int linkid;
+	    , int linkid)
 #else /* SHORTLINKS */
      )
 #endif /* SHORTLINKS */
-     int id;
-     int ins;
 {
 	u_int bucket;
 	cfs_fileid *f;
@@ -395,10 +349,8 @@ findh(id,ins
 	return NULL;
 }
 
-
 cfs_fileid *
-geth(h)
-     fhdata *h;	/* note recasting here - may give trouble with strict ansi */
+geth(nfs_fh *h)
 {
 	int i;
 	cfs_fileid *r;
@@ -407,33 +359,33 @@ geth(h)
 		return &rootnode;
 	/* check for rootness */
 	for (i=0; i<8; i++)
-		if (h->magic[i] != magictest[i]) {
+		if (FHDATA_MAGIC(h)[i] != magictest[i]) {
 			/* update the root handle, so we return the
 			   latest thing it gave out for lookups of /.  */
 			bcopy((char *)h,(char*)&roothandle,sizeof(roothandle));
 			return &rootnode;
 		}
-	if (instances[h->instance] == NULL)
+	if (instances[FHDATA_INSTANCE(h)] == NULL)
 		return NULL;	/* invalid/stale instance */
-	r=findh(h->fileid,h->instance
+	r = findh(FHDATA_FILEID(h), FHDATA_INSTANCE(h)
 #ifdef SHORTLINKS
-				     ,h->linkid
+				     ,FHDATA_LINKID(h)
 #endif /* SHORTLINKS */
 				     );
 	/* instance may have timed out */
 	if ((r != NULL) && (r->ins->dead != 0)) {
 		/* if so, we GC it first */
-		freeinstance(h->instance);
+		freeinstance(FHDATA_INSTANCE(h));
 		r=NULL;
 	}
-	if ((r==NULL) || (bcmp(r->ins->check,h->check,8)!=0))
+	if ((r == NULL) || (bcmp(r->ins->check, FHDATA_CHECK(h), 8) != 0))
 		return NULL;	/* cheater */
 	return r;
 }
 
 
-htype(f)
-     cfs_fileid *f;
+int
+htype(cfs_fileid *f)
 {
 	if (f == &rootnode)
 		return H_ROOT;
@@ -443,8 +395,8 @@ htype(f)
 		return H_REG;
 }
 
-iid(f)
-     cfs_fileid *f;
+int
+iid(cfs_fileid *f)
 {
 	if (f == &rootnode)
 		return -1;
@@ -454,18 +406,16 @@ iid(f)
 		return f->ins->id;
 }
 
-fhowner(f,uid)
-     cfs_fileid *f;
-     int uid;
+int
+fhowner(cfs_fileid *f, int uid)
 {
 	if (f->fileid == 0)	/* an instance root */
 		return (f->ins->uid);	/* return the instance owner */
 	return (uid);	/* normal case */
 }
 
-fhmode(f,mode)
-     cfs_fileid *f;
-     int mode;
+int
+fhmode(cfs_fileid *f, int mode)
 {
 	if (f->fileid == 0)	/* an instance root */
 		return (mode&(~077));	/* set a umask of 0077 */
@@ -473,8 +423,8 @@ fhmode(f,mode)
 }
 
 /* return the inode of me */
-fhid(f)
-     cfs_fileid *f;
+int
+fhid(cfs_fileid *f)
 {
 	if (f==NULL)
 		return 1;
@@ -484,23 +434,24 @@ fhid(f)
 }
 
 /* return the inode of .. (directory only) */
-fhpid(f)
-     cfs_fileid *f;
+int
+fhpid(cfs_fileid *f)
 {
 	if (f==NULL)	/* this is main root */
 		return 1;
-	if (f->parent<=0) /* child of instance root */
-		if (f->fileid==0)
+	if (f->parent<=0) {	/* child of instance root */
+		if (f->fileid==0) {
 			return 1;
-		else
+		} else {
 			return (0-f->ins->id);
+		}
+	}
 	return f->parent;	/* this is already 2 for main root  */
 }
 
 /* is uid authorized for this? */
-fhuid(f,u)
-     cfs_fileid *f;
-     int u;
+int
+fhuid(cfs_fileid *f, int u)
 {
 	if (f==NULL)
 		return 1;
@@ -511,8 +462,8 @@ fhuid(f,u)
 	return (f->ins->uid==u);
 }
 
-cfsno(err)
-     int err;
+int
+cfsno(int err)
 {
 /* 	if (err==ENOENT)
 		return NFSERR_STALE;
@@ -527,9 +478,7 @@ static int curfd= -1;
 
 /* open file descriptor for handle.  maintains cache of one fd */
 int
-fhtofd(f,mode)
-     cfs_fileid *f;
-     int mode;
+fhtofd(cfs_fileid *f, int mode)
 {
 	int fd;
 
@@ -580,8 +529,7 @@ fhtofd(f,mode)
 }
 
 int
-modeof(s)
-     char *s;
+modeof(char *s)
 {
 	struct stat sb;
 
@@ -591,9 +539,7 @@ modeof(s)
 }
 
 int
-fhgetstat(h,sb)
-     cfs_fileid *h;
-     struct stat *sb;
+fhgetstat(cfs_fileid *h, struct stat *sb)
 {
 	if (h == NULL) {
 		cfserrno=NFSERR_STALE;
@@ -607,8 +553,8 @@ fhgetstat(h,sb)
 	return 0;
 }
 
-closeout(f)
-     cfs_fileid *f;
+void
+closeout(cfs_fileid *f)
 {
 	if (openfd==f) {
 		if (curfd>=0)
@@ -618,7 +564,8 @@ closeout(f)
 	}
 }
 
-closeall()
+void
+closeall(void)
 {
 	if (openfd!=NULL) {
 		if (curfd>=0)
@@ -628,9 +575,8 @@ closeall()
 	}
 }
 
-fhsetattr(f,a)
-     cfs_fileid *f;
-     sattr *a;
+int
+fhsetattr(cfs_fileid *f, sattr *a)
 {
 	struct timeval tv[2];
 	struct stat sb;
@@ -680,9 +626,7 @@ fhsetattr(f,a)
 }
 
 int
-fhsetattrprime(f,a)
-     cfs_fileid *f;
-     sattr *a;
+fhsetattrprime(cfs_fileid *f, sattr *a)
 {
 	struct timeval tv[2];
 	struct stat sb;
@@ -725,10 +669,8 @@ fhsetattrprime(f,a)
 	return 0;
 }
 
-fhmkdirent(p,comp,h)
-     cfs_fileid *p;
-     char *comp;
-     fhdata *h;
+int
+fhmkdirent(cfs_fileid *p, char * comp, nfs_fh *h)
 {
 	char path[NFS_MAXPATHLEN+1];
 	struct stat sb;
@@ -750,10 +692,7 @@ fhmkdirent(p,comp,h)
 }
 
 int
-fhmkfileent(p,comp,h)
-     cfs_fileid *p;
-     char *comp;
-     fhdata *h;
+fhmkfileent(cfs_fileid *p, char *comp, nfs_fh *h)
 {
 	char path[NFS_MAXPATHLEN+1];
 	int fd;
@@ -763,7 +702,7 @@ fhmkfileent(p,comp,h)
 	char vect[9];
 	union{
 		u_char ch[9];
-		u_long i[2];
+		uint32_t i[2];
 	} buf;
 	char linkname[NFS_MAXPATHLEN+1];
 	
@@ -792,9 +731,9 @@ fhmkfileent(p,comp,h)
 		/*
 		sprintf((char *)buf,"%08x",(u_long)sb.st_ino+(u_long)sb.st_ctime);
 		*/
-		buf.i[0]=(u_long)sb.st_ino;
-		buf.i[1]=(u_long)sb.st_ctime;
-		q_block_cipher("fixedkey",&buf,1);
+		buf.i[0]=(uint32_t)sb.st_ino;
+		buf.i[1]=(uint32_t)sb.st_ctime;
+		q_block_cipher((u_char *)"fixedkey", buf.ch, 1);
 		/* des is just used here as a hash fn to spread the bits */
 		/* since we only use 32 bits of the result, its a nonperfect */
 		/* hash. but this doesn't really matter since collisions */
@@ -830,34 +769,47 @@ fhmkfileent(p,comp,h)
 
 
 int
-fhlook(p,comp,h)
-     cfs_fileid *p;
-     char *comp;
-     fhdata *h;
+fhlook(cfs_fileid *p, char *comp, nfs_fh *h)
 {
 	char path[NFS_MAXPATHLEN+1];
 	char linkname[NFS_MAXPATHLEN+1];
 	char vect[NFS_MAXPATHLEN+1];
 	struct stat sb;
+	int l;
 	
 	if (p==NULL) {
 		cfserrno=NFSERR_STALE;
 		return -2;
 	}
-	if (p->fileid==0) {
-		if (!strcmp(comp,"."))
-			return fhrootlook(p->ins->name,h);
-		if (!strcmp(comp,".."))
-			return fhrootlook(".",h);
+	if (comp == NULL) {
+		cfserrno = NFSERR_NOENT;
+		return (-2);
+	}
+	/* Check for "." and ".." */
+	if (p->fileid == 0) {
+		if (comp[0] == '.') {
+			if (comp[1] == '\0')
+				return (fhrootlook(p->ins->name, h));
+			else if (comp[1] == '.' && comp[2] == '\0')
+				return (fhrootlook(".", h));
+		}
+	}
+	if ((p->parent == 0) && (strncmp(comp, "..", 3) == 0))
+		return (fhrootlook(p->ins->name, h));
+	l = snprintf(path, NFS_MAXPATHLEN, "%s/%s", p->name, comp);
+	if (l < 0 || l > NFS_MAXPATHLEN) {
+		cfserrno = NFSERR_NAMETOOLONG;
+		return -2;
 	}
-	if ((p->parent==0) && (!strcmp(comp,"..")))
-		return fhrootlook(p->ins->name,h);
-	sprintf(path,"%s/%s",p->name,comp);
 	if (lstat(path,&sb)<0) {
 		return -1;	/* just need the inode */
 	}
 	/* check for a linkfile */
-	sprintf(linkname,"%s/.pvect_%s",p->name,comp);
+	l = snprintf(linkname, NFS_MAXPATHLEN, "%s/.pvect_%s", p->name, comp);
+	if (l < 0 || l > NFS_MAXPATHLEN) {
+		cfserrno = NFSERR_NAMETOOLONG;
+		return -2;
+	}
 	if (readlink(linkname,vect,9) != 8) {
 		bcopy((char *)zerovect,(char *)vect,8);
 		strcpy(linkname,"/NOWHERE/null");
@@ -881,21 +833,15 @@ fhlook(p,comp,h)
 
 
 int
-mkhandle(p,n,comp,ino,h,vect,vectname
+mkhandle(cfs_fileid *p, char *n, char *comp,
+    int ino, nfs_fh *h, char *vect, char *vectname
 #ifdef SHORTLINKS
-				     ,isl)
-     int isl;	/* is short link */
+				     ,int isl)
 #else /* SHORTLINKS */
 				     )
 #endif /* SHORTLINKS */
-     cfs_fileid *p;
-     char *n;
-     char *comp;
-     int ino;
-     fhdata *h;
-     char *vect;
-     char *vectname;
 {
+	int l;
 	cfs_fileid *f;
 
 #ifdef SHORTLINKS
@@ -903,12 +849,12 @@ mkhandle(p,n,comp,ino,h,vect,vectname
 #else /* SHORTLINKS */
 	if ((f=findh(ino,p->ins->id)) != NULL) {
 #endif /* SHORTLINKS */
-		if (strcmp(comp,".") && strcmp(comp,"..")) {
+		if (!is_dot_or_dotdot(comp)) {
 			free(f->name);
 			f->name=NULL;
 		}
 	} else {
-		if ((f=(cfs_fileid *)malloc(sizeof(cfs_fileid)))==NULL) {
+		if ((f=(cfs_fileid *)calloc(1, sizeof(cfs_fileid)))==NULL) {
 			fprintf(stderr,"cfsd: out of memory\n");
 			cfserrno=NFSERR_STALE;	/* bad news */
 			return -2;
@@ -932,24 +878,29 @@ mkhandle(p,n,comp,ino,h,vect,vectname
 			cfserrno=NFSERR_STALE;
 			return -2;
 		}
-		strcpy(f->name,n);
-		strcpy(f->vectname,vectname);
+		strcpy(f->name, n); /* Just malloced! */
+		l = snprintf(f->vectname, sizeof(f->vectname), "%s", vectname);
+		if (l < 0 || l >= sizeof(f->vectname)) {
+			DEBUGMSG("mkhandle: vectname too long for f->vectname: %s", vectname);
+			cfserrno = NFSERR_NAMETOOLONG;
+			return (-2);
+		}
 		bcopy((char *)vect,(char *)f->vect,8);
 		f->vect[8]='\0';
 	}
 	if (h!=NULL) {
 		fillinh(h,p->ins->id,ino,p->ins->check);
 #ifdef SHORTLINKS
-		h->linkid = f->linkid;
+		FHDATA_LINKID(h) = f->linkid;
 		if (isl)
-			h->htype = H_SLNK;
+			FHDATA_HTYPE(h) = H_SLNK;
 #endif /* SHORTLINKS */
 	}
 	return 0;
 }
 
-inserth(f)
-     cfs_fileid *f;
+void
+inserth(cfs_fileid *f)
 {
 	u_int bucket;
 	
@@ -958,24 +909,19 @@ inserth(f)
 	instances[f->ins->id]->file[bucket]=f;
 }
 
-fillinh(h,iid,ino,check)
-     fhdata *h;
-     int iid;
-     int ino;
-     char *check;
+void
+fillinh(nfs_fh *h, int iid, int ino, u_char *check)
 {
-	bzero((char *)h,sizeof(fhdata));
-	bcopy((char *)magictest,(char *)h->magic,sizeof(magictest));
-	bcopy((char *)check,(char *)h->check,8);
-	h->htype=H_REG;
-	h->instance=iid;
-	h->fileid=ino;
+	bzero((char *)h, sizeof(nfs_fh));
+	bcopy((char *)magictest, (char *)FHDATA_MAGIC(h), sizeof(magictest));
+	bcopy((char *)check, (char *)FHDATA_CHECK(h), 8);
+	FHDATA_HTYPE(h) = H_REG;
+	FHDATA_INSTANCE(h) = iid;
+	FHDATA_FILEID(h) = ino;
 }
 
 int
-fhdelete(f,s)
-     cfs_fileid *f;
-     char *s;
+fhdelete(cfs_fileid *f, char *s)
 {
 	char path[NFS_MAXPATHLEN+1];
 	char linkname[NFS_MAXPATHLEN+1];
@@ -996,9 +942,7 @@ fhdelete(f,s)
 }
 
 int
-fhdeletedir(f,s)
-     cfs_fileid *f;
-     char *s;
+fhdeletedir(cfs_fileid *f, char *s)
 {
 	char path[NFS_MAXPATHLEN+1];
 
@@ -1011,8 +955,7 @@ fhdeletedir(f,s)
 }
 
 DIR *
-fhopendir(d)
-     cfs_fileid *d;
+fhopendir(cfs_fileid *d)
 {
 	DIR *dp;
 	
@@ -1025,17 +968,15 @@ fhopendir(d)
 	return dp;
 }
 
-fhclosedir(dp)
-     DIR *dp;
+void
+fhclosedir(DIR *dp)
 {
 	closedir(dp);
 }
 
 
 int
-fhlinkval(f,buf)
-     cfs_fileid *f;
-     char *buf;
+fhlinkval(cfs_fileid *f, char *buf)
 {
 	if (f==NULL) {
 		cfserrno=NFSERR_STALE;
@@ -1046,10 +987,7 @@ fhlinkval(f,buf)
 
 
 int
-fhdohardlink(f,t,n)
-     cfs_fileid *f;
-     cfs_fileid *t;
-     char *n;
+fhdohardlink(cfs_fileid *f, cfs_fileid *t, char *n)
 {
 	char buf[NFS_MAXPATHLEN+1];
 	char linkname[NFS_MAXPATHLEN+1];
@@ -1073,10 +1011,7 @@ fhdohardlink(f,t,n)
 }
 
 int
-fhdosymlink(f,n,t)
-     cfs_fileid *f;
-     char *n;
-     char *t;
+fhdosymlink(cfs_fileid *f, char *n, char *t)
 {
 	char buf[NFS_MAXPATHLEN+1];
 	
@@ -1089,11 +1024,7 @@ fhdosymlink(f,n,t)
 }
 
 int
-fhrename(f,fn,t,tn)
-     cfs_fileid *f;
-     char *fn;
-     cfs_fileid *t;
-     char *tn;
+fhrename(cfs_fileid *f, char *fn, cfs_fileid *t, char *tn)
 {
 	char fb[NFS_MAXPATHLEN+1];
 	char tb[NFS_MAXPATHLEN+1];
@@ -1124,8 +1055,7 @@ fhrename(f,fn,t,tn)
 }
 
 int
-getftype(m)
-     int m;
+getftype(int m)
 {
 	if ((m&S_IFMT) == S_IFREG)
 		return NFREG;
@@ -1145,8 +1075,7 @@ getftype(m)
 /* should make . and .. the first entries, but this probably doesn't
    break anything */
 struct dirent *
-rootrd(cookie)
-     long cookie;
+rootrd(int32_t cookie)
 {
 	static struct dirent d;
 
@@ -1177,28 +1106,27 @@ rootrd(cookie)
 
 
 int
-fhrootlook(n,h)
-     char *n;
-     nfs_fh *h;
+fhrootlook(char *n, nfs_fh *h)
 {
-	int i;
+	int i, l;
 	cfs_fileid *f;
 	
-	if (!strcmp(n,".") || !strcmp(n,"..")) {	/* self */
+	if (is_dot_or_dotdot(n)) {	/* self */
 		bcopy((char *)&roothandle,(char *)h,sizeof(roothandle));
 		return 0;
 	}
 	for (i=0; i<NINSTANCES; i++) {
 		if (instances[i]==NULL)
 			continue;
-		if (!strcmp(instances[i]->name,n)) {
+		if (strncmp(instances[i]->name, n,
+				sizeof(instances[i]->name)) == 0) {
 			if ((f=findh(0,i
 #ifdef SHORTLINKS
 					,0
 #endif /* SHORTLINKS */
 					)) == NULL) {
 				if ((f=(cfs_fileid *)
-				     malloc(sizeof(cfs_fileid)))==NULL) {
+				     calloc(1, sizeof(cfs_fileid)))==NULL) {
 					fprintf(stderr,"cfsd: no memory\n");
 					cfserrno=NFSERR_STALE;	/* bad news */
 					return -2;
@@ -1214,7 +1142,13 @@ fhrootlook(n,h)
 					cfserrno=NFSERR_STALE;
 					return -2;
 				}
-				strcpy(f->name,instances[i]->path);
+				l = snprintf(f->name, NFS_MAXPATHLEN, "%s",
+						instances[i]->path);
+				if (l < 0 || l >= NFS_MAXPATHLEN) {
+					DEBUGMSG("fhrootlook: instances[%d]->path too long for f->name: %s", i, instances[i]->path);
+					cfserrno = NFSERR_NAMETOOLONG;
+					return (-2);
+				}
 			}
 			fillinh(h,i,0,f->ins->check);
 			return 0;
