@@ -21,46 +21,45 @@
  *  access control policies
  */
 
-#include <stdio.h>
-#include <sys/file.h>
 #include <sys/types.h>
+#include <sys/file.h>
 #include <sys/stat.h>
+
 #include <rpc/rpc.h>
-#include <errno.h>
-#ifdef NeXT
-#include <sys/dir.h>
-#define dirent direct
-#else
+
 #include <dirent.h>
-#endif
+#include <errno.h>
+#include <stdio.h>
+#include <string.h>
+
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
 
-typedef struct svc_req *SR;
-
 #define herr(x) (((x)==H_INVALID)?NFSERR_STALE:NFSERR_PERM)
 
+/* Return a string dumping the filehandle */
 char *
-pfh(fh)
-     unsigned char *fh;
+pfh(nfs_fh *fh)
 {
 	static char ret[NFS_FHSIZE*2+2];
-	unsigned char x[3];
+	u_char *data;
+	char x[3];
 	int i;
 
-	ret[0]='\0';
+	data = (u_char *)fh;
+
 	for (i=0; i<NFS_FHSIZE; i++) {
-		sprintf((char *)x,"%02x",fh[i]);
-		strcat(ret,x);
+		sprintf(x, "%02x", data[i]); /* Cannot overflow */
+		ret[2*i] = x[0];
+		ret[2*i+1] = x[1];
 	}
+	ret[2*i+2] = '\0';
 	return ret;
 }
 
 void *
-nfsproc_null_2(ap,rp)
-     void *ap;
-     SR rp;
+nfsproc_null_2_svc(void *ap, SR rp)
 {
 	static int ret=0;
 	return ((void*) &ret);
@@ -68,13 +67,8 @@ nfsproc_null_2(ap,rp)
 
 #define as_attr attrstat_u.attributes
 
-diropres *rootlookup();
-readdirres *rootreaddir();
-
 attrstat *
-nfsproc_getattr_2(ap,rp)
-     nfs_fh *ap;
-     SR rp;
+nfsproc_getattr_2_svc(nfs_fh *ap, SR rp)
 {
 	static attrstat ret;
 	int uid;
@@ -101,7 +95,7 @@ nfsproc_getattr_2(ap,rp)
 		return &ret;
 	}
 	become(rp);	/* become the user */
-	switch (fhstat(h,&ret.as_attr)) {
+	switch (cfs_fhstat(h,&ret.as_attr)) {
 	    case 0:	/* ok */
 		ret.status=NFS_OK;
 		break;
@@ -118,9 +112,7 @@ nfsproc_getattr_2(ap,rp)
 }
 
 attrstat *
-nfsproc_setattr_2(ap,rp)
-     sattrargs *ap;
-     SR rp;
+nfsproc_setattr_2_svc(sattrargs *ap, SR rp)
 {
 	static attrstat ret;
 	int uid;
@@ -143,7 +135,7 @@ nfsproc_setattr_2(ap,rp)
 	switch (fhsetattr(h,&ap->attributes)) {
 	    case 0:	/* ok */
 		/* now do a getattr */		
-		switch (fhstat(h,&ret.as_attr)) {
+		switch (cfs_fhstat(h,&ret.as_attr)) {
 		    case 0:
 			ret.status=NFS_OK;
 			break;
@@ -168,9 +160,7 @@ nfsproc_setattr_2(ap,rp)
 }
 
 void *
-nfsproc_root_2(ap,rp)
-     void *ap;
-     SR rp;
+nfsproc_root_2_svc(void *ap, SR rp)
 {
 	static int ret=0;
 
@@ -179,9 +169,7 @@ nfsproc_root_2(ap,rp)
 
 /* fix this to deal w/ fs root (instance root should be ok) */
 diropres *
-nfsproc_lookup_2(ap,rp)
-     diropargs *ap;
-     SR rp;
+nfsproc_lookup_2_svc(diropargs *ap, SR rp)
 {
 	static diropres ret;
 	diropres *retp;
@@ -215,7 +203,7 @@ nfsproc_lookup_2(ap,rp)
 		return &ret;
 	}
 	uid=ruid(rp);
- 	if (!fhuid(h,uid)) {
+	if (!fhuid(h,uid)) {
 		ret.status = NFSERR_PERM;
 		return &ret;
 	}
@@ -232,7 +220,7 @@ nfsproc_lookup_2(ap,rp)
 	       handle, or delete the handle */
 	    case 0:	/* found it */
 		/* now do a getattr */		
-		switch (fhstat(geth(handle),
+		switch (cfs_fhstat(geth(handle),
 			       &ret.diropres_u.diropres.attributes)) {
 		    case 0:
 			ret.status=NFS_OK;
@@ -258,9 +246,7 @@ nfsproc_lookup_2(ap,rp)
 }
 
 readlinkres *
-nfsproc_readlink_2(ap,rp)
-     nfs_fh *ap;
-     SR rp;
+nfsproc_readlink_2_svc(nfs_fh *ap, SR rp)
 {
 	static readlinkres ret;
 	int uid;
@@ -309,9 +295,7 @@ nfsproc_readlink_2(ap,rp)
 
 
 readres *
-nfsproc_read_2(ap,rp)
-     readargs *ap;
-     SR rp;
+nfsproc_read_2_svc(readargs *ap, SR rp)
 {
 	static readres ret;
 	static char buffer[8192];
@@ -354,7 +338,7 @@ nfsproc_read_2(ap,rp)
 			ret.status = cfsno(errno);
 			break;
 		}
-		if (fhstat(h,&ret.readres_u.reply.attributes)!=0) {
+		if (cfs_fhstat(h,&ret.readres_u.reply.attributes)!=0) {
 			ret.status = cfsno(errno);
 			break;
 		}
@@ -370,9 +354,7 @@ nfsproc_read_2(ap,rp)
 
 
 void *
-nfsproc_writecache_2(ap,rp)
-     void *ap;
-     SR rp;
+nfsproc_writecache_2_svc(void *ap, SR rp)
 {
 	static int ret;
 
@@ -381,9 +363,7 @@ nfsproc_writecache_2(ap,rp)
 
 
 attrstat *
-nfsproc_write_2(ap,rp)
-     writeargs *ap;
-     SR rp;
+nfsproc_write_2_svc(writeargs *ap, SR rp)
 {
 	static attrstat ret;
 	int fd;
@@ -429,7 +409,7 @@ nfsproc_write_2(ap,rp)
 			ret.status = cfsno(errno);
 			break;
 		}
-		if (fhstat(h,&ret.attrstat_u.attributes)!=0) {
+		if (cfs_fhstat(h,&ret.attrstat_u.attributes)!=0) {
 			ret.status = cfsno(errno);
 			break;
 		}
@@ -442,9 +422,7 @@ nfsproc_write_2(ap,rp)
 
 
 diropres *
-nfsproc_create_2(ap,rp)
-     createargs *ap;
-     SR rp;
+nfsproc_create_2_svc(createargs *ap, SR rp)
 {
 	static diropres ret;
 	static nfs_fh handle;
@@ -482,7 +460,7 @@ nfsproc_create_2(ap,rp)
 		switch (fhsetattr(h2=geth(&handle),&ap->attributes)) {
 		    case 0:	/* ok */
 			/* now do a getattr */		
-			switch (fhstat(h2,
+			switch (cfs_fhstat(h2,
 				       &ret.diropres_u.diropres.attributes)) {
 			    case 0:
 				bcopy(&handle,&ret.diropres_u.diropres.file,
@@ -509,7 +487,7 @@ nfsproc_create_2(ap,rp)
 		switch (fhsetattrprime(h2=geth(&handle),&ap->attributes)) {
 		    case 0:	/* ok */
 			/* now do a getattr */		
-			switch (fhstat(h2,
+			switch (cfs_fhstat(h2,
 				       &ret.diropres_u.diropres.attributes)) {
 			    case 0:
 				bcopy(&handle,&ret.diropres_u.diropres.file,
@@ -545,9 +523,7 @@ nfsproc_create_2(ap,rp)
 
 
 nfsstat *
-nfsproc_remove_2(ap,rp)
-     diropargs *ap;
-     SR rp;
+nfsproc_remove_2_svc(diropargs *ap, SR rp)
 {
 	static nfsstat ret;
 	char *s;
@@ -587,9 +563,7 @@ nfsproc_remove_2(ap,rp)
 
 
 nfsstat *
-nfsproc_rename_2(ap,rp)
-     renameargs *ap;
-     SR rp;
+nfsproc_rename_2_svc(renameargs *ap, SR rp)
 {
 	static nfsstat ret;
 	int uid;
@@ -619,10 +593,10 @@ nfsproc_rename_2(ap,rp)
 	if ((key=keyof(fromh)) == NULL)
 		ret=cfserrno;
 	else if (((s=encryptname(key,ap->from.name)) == NULL) ||
-		 (strcpy(s1,s)==NULL))
+		 (strcpy(s1,s)==NULL)) /* Both defined [NFS_MAXNAMELEN+1] */
 		ret=cfserrno;
 	else if (((s=encryptname(key,ap->to.name)) == NULL) ||
-		 (strcpy(s2,s)==NULL))
+		 (strcpy(s2,s)==NULL)) /* Both defined [NFS_MAXNAMELEN+1] */
 		ret=cfserrno;
 	/* if s1 already has a handle, rename updates its underlying name */
 	else switch (fhrename(fromh,s1,
@@ -643,9 +617,7 @@ nfsproc_rename_2(ap,rp)
 
 
 nfsstat *
-nfsproc_link_2(ap,rp)
-     linkargs *ap;
-     SR rp;
+nfsproc_link_2_svc(linkargs *ap, SR rp)
 {
 	static nfsstat ret;
 	int uid;
@@ -694,9 +666,7 @@ nfsproc_link_2(ap,rp)
 
 
 nfsstat *
-nfsproc_symlink_2(ap,rp)
-     symlinkargs *ap;
-     SR rp;
+nfsproc_symlink_2_svc(symlinkargs *ap, SR rp)
 {
 	static nfsstat ret;
 	int uid;
@@ -722,9 +692,9 @@ nfsproc_symlink_2(ap,rp)
 	if ((s=encryptname(key,ap->from.name))==NULL)
 		ret=cfserrno;
 	else {
-		strcpy(s1,s);
+		strcpy(s1,s);	/* Both defined as [NFS_MAXNAMLEN+1] */
 		s=encryptname(key,ap->to); /* can't fail */
-		strcpy(s2,s);
+		strcpy(s2,s);	/* Both defined as [NFS_MAXNAMLEN+1] */
 		/* ignore the attributes stuff */
 		switch (fhdosymlink(h,s1,s2)) {
 		    case -1:
@@ -743,9 +713,7 @@ nfsproc_symlink_2(ap,rp)
 }
 
 diropres *
-nfsproc_mkdir_2(ap,rp)
-     createargs *ap;
-     SR rp;
+nfsproc_mkdir_2_svc(createargs *ap, SR rp)
 {
 	static diropres ret;
 	nfs_fh handle;
@@ -781,7 +749,7 @@ nfsproc_mkdir_2(ap,rp)
 		switch (fhsetattr(h2=geth(&handle),&ap->attributes)) {
 		    case 0:	/* ok */
 			/* now do a getattr */		
-			switch (fhstat(h2=geth(&handle),
+			switch (cfs_fhstat(h2=geth(&handle),
 				       &ret.diropres_u.diropres.attributes)) {
 			    case 0:
 				bcopy(&handle,&ret.diropres_u.diropres.file,
@@ -817,9 +785,7 @@ nfsproc_mkdir_2(ap,rp)
 
 
 nfsstat *
-nfsproc_rmdir_2(ap,rp)
-     diropargs *ap;
-     SR rp;
+nfsproc_rmdir_2_svc(diropargs *ap, SR rp)
 {
 	static nfsstat ret;
 	char *s;
@@ -864,9 +830,7 @@ nfsproc_rmdir_2(ap,rp)
 /* #define cfsclosedir(x) fhclosedir(x) */
 
 readdirres *
-nfsproc_readdir_2(ap,rp)
-     readdirargs *ap;
-     SR rp;
+nfsproc_readdir_2_svc(readdirargs *ap, SR rp)
 {
 	static readdirres ret;
 	int uid;
@@ -877,15 +841,12 @@ nfsproc_readdir_2(ap,rp)
 	static DIR *dp=NULL;
 	static struct dirent *dent;
 	entry **prev;
-	long loc;
 	char s1[NFS_MAXNAMLEN+1];
 	char *s;
 	cfskey *key;
-	static long curcookie=0;
-	long cookie;
-	int eof;
-	int ne;
-	int bytes;
+	static int32_t curcookie=0;
+	int32_t cookie;
+	int l, eof, ne, bytes;
 	DIR *cfsopendir();
 	cfs_fileid *h;
 	static int fid= -10;
@@ -937,14 +898,21 @@ nfsproc_readdir_2(ap,rp)
 		if (((bytes -= dentsize(s)) < 0) || (ne >= MAXENTRIES))
 			break;
 		eof=TRUE;
-		strcpy(names[ne],s);
+		l = snprintf(names[ne], sizeof(names[ne]), "%s", s);
+		if (l < 0 || l >= sizeof(names[ne])) {
+			ret.status = NFSERR_NAMETOOLONG;
+			break;
+		}
 		entrytab[ne].name=names[ne];
-		if (strcmp(s,".")==0)	/* me */
-			entrytab[ne].fileid=fhid(h);
-		else if (strcmp(s,"..")==0)	/* parent */
-			entrytab[ne].fileid=fhpid(h);
-		else entrytab[ne].fileid=dent->d_fileno;
-		bcopy(&cookie,entrytab[ne].cookie,sizeof(long));
+		/* . or .. ? */
+		if (s[0] == '.') {				/* me */
+			if (s[1] == '\0')
+				entrytab[ne].fileid=fhid(h);
+			else if (s[1] == '.' && s[2] == '\0')	/* parent */
+				entrytab[ne].fileid=fhpid(h);
+		} else
+			entrytab[ne].fileid=dent->d_fileno;
+		bcopy(&cookie,entrytab[ne].cookie,sizeof(int32_t));
 		*prev = &entrytab[ne];
 		prev = &entrytab[ne].nextentry;
 		entrytab[ne].nextentry=NULL;
@@ -964,12 +932,10 @@ nfsproc_readdir_2(ap,rp)
 
 DIR *curdir;
 int curdirid=0;
-long curcookie;
+int32_t curcookie;
 
 DIR *
-cfsopendir(dir,cookie)
-     cfs_fileid *dir;
-     long cookie;
+cfsopendir(cfs_fileid *dir, int32_t cookie)
 {
 	DIR *ret;
 	DIR *fhopendir();
@@ -984,16 +950,14 @@ cfsopendir(dir,cookie)
 	return ret;
 }
 
-cfsclosedir(dp)
-     DIR *dp;
+void
+cfsclosedir(DIR *dp)
 {
 	fhclosedir(dp);
 }
 
 statfsres *
-nfsproc_statfs_2(ap,rp)
-     nfs_fh *ap;
-     SR rp;
+nfsproc_statfs_2_svc(nfs_fh *ap, SR rp)
 {
 	static statfsres ret;
 
@@ -1005,8 +969,8 @@ nfsproc_statfs_2(ap,rp)
 	return (&ret);
 }
 
-rootgetattr(f)
-	struct fattr *f;
+void
+rootgetattr(struct fattr *f)
 {
 	f->type=NFDIR;
 	f->mode=NFSMODE_DIR|0777;
@@ -1025,8 +989,8 @@ rootgetattr(f)
 	
 }
 
-setstatfsokres(s)
-	statfsokres *s;
+void
+setstatfsokres(statfsokres *s)
 {
 	s->tsize=8192;
 	s->bsize=8192;
@@ -1035,9 +999,8 @@ setstatfsokres(s)
 	s->bavail=0;
 }
 
-int fhstat(fh,fa)
-     cfs_fileid *fh;
-     fattr *fa;
+int
+cfs_fhstat(cfs_fileid *fh, fattr *fa)
 {
 	struct stat sb;
 	int r;
@@ -1078,8 +1041,7 @@ int fhstat(fh,fa)
 /* return uid associated w/ rpc call - unix style only for now */
 /* note that this really doesnt belong here */
 int
-ruid(rp)
-     SR rp;
+ruid(SR rp)
 {
 	struct authunix_parms *cred;
 
@@ -1090,8 +1052,7 @@ ruid(rp)
 }
 
 int
-rgid(rp)
-     SR rp;
+rgid(SR rp)
 {
 	struct authunix_parms *cred;
 
@@ -1104,8 +1065,7 @@ rgid(rp)
 /* this is ugly, since it duplicates much of the code
    above in lookup, but it does the job */
 diropres *
-rootlookup(s)
-     char *s;
+rootlookup(char *s)
 {
 	static diropres ret;
 	nfs_fh *handle;
@@ -1115,7 +1075,7 @@ rootlookup(s)
 	    case 0:	/* found it */
 		/* now do a getattr - handle is an instance root, which
 		   is treated normally */
-		switch (fhstat(geth(handle),
+		switch (cfs_fhstat(geth(handle),
 			       &ret.diropres_u.diropres.attributes)) {
 		    case 0:
 			ret.status=NFS_OK;
@@ -1142,19 +1102,15 @@ rootlookup(s)
 /* ugly ugly ugly - duplicates fair bit of code from readdir, but at least
    it works */
 readdirres *
-rootreaddir(ap)
-    readdirargs *ap;
-
+rootreaddir(readdirargs *ap)
 {
 	static readdirres ret;
 	static entry entrytab[MAXENTRIES];	/* just deal w/ it staticaly */
 	typedef char str[NFS_MAXNAMLEN+1];
 	static str names[MAXENTRIES];
 	entry **prev;
-	long cookie;
-	int eof;
-	int ne;
-	int bytes;
+	int32_t cookie;
+	int l, eof, ne, bytes;
 	cfs_fileid *h;
 	struct dirent *dent;
 	struct dirent *rootrd();
@@ -1163,7 +1119,7 @@ rootreaddir(ap)
 	ne=0;
 	prev= &ret.readdirres_u.reply.entries;
 	*prev=NULL;
-	bcopy(ap->cookie,&cookie,sizeof(long));
+	bcopy(ap->cookie,&cookie,sizeof(int32_t));
 	eof=1;
 	
 	ret.status=NFS_OK;
@@ -1173,16 +1129,19 @@ rootreaddir(ap)
 			break;
 		}
 		eof=1;
-		strcpy(names[ne],dent->d_name);
+		l = snprintf(names[ne], sizeof(names[ne]), "%s", dent->d_name);
+		if (l < 0 || l >= sizeof(names[ne])) {
+			ret.status = NFSERR_NAMETOOLONG;
+			break;
+		}
 		entrytab[ne].name=names[ne];
-		if (strcmp(dent->d_name,".")==0)	/* me */
-			entrytab[ne].fileid=1;
-		else if (strcmp(dent->d_name,"..")==0) /* parent, also me */
-			entrytab[ne].fileid=1;
-		else entrytab[ne].fileid=dent->d_fileno;
+		if (dent->d_name[0] == '.' && dent->d_name[1] == '\0')
+			entrytab[ne].fileid = 1;
+		else
+			entrytab[ne].fileid = dent->d_fileno;
 		cookie=dent->d_reclen;	/* may not work everywhere */
 		*prev = &entrytab[ne];
-		bcopy(&cookie,entrytab[ne].cookie,sizeof(long));
+		bcopy(&cookie,entrytab[ne].cookie,sizeof(int32_t));
 		prev = &entrytab[ne].nextentry;
 		entrytab[ne].nextentry=NULL;
 		ne++;
@@ -1193,8 +1152,8 @@ rootreaddir(ap)
 
 extern struct in_addr validhost;
 
-goodsrc(rp)
-     SR rp;
+int
+goodsrc(SR rp)
 {
 	struct in_addr ia;
 	struct in_addr *ip;
