@@ -18,37 +18,41 @@
 /*
  * client cfs mkdir - 1.4.0
  */
-#include <stdio.h>
-#include <rpc/rpc.h>
 #include <sys/time.h>
+
+#include <rpc/rpc.h>
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
+#include "getpass.h"
 #include "shs.h"
+#include "truerand.h"
 
-main(argc,argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char *argv[])
 {
 	char *pw;
-	char pword[256];
-	char *getpassword();
+	char pword[MAX_PASSPHRASE_LEN+1];
 	int n;
 	cfs_admkey k;
 	cfskey kt;
-	char path[1024];
+	char path[NFS_MAXPATHLEN+1];
 	char str[8];
 	FILE *fp;
 	char *flg;
 	struct timeval tv;
 	u_long r;
-	int i;
-	int ciph=CFS_THREE_DES;
+	int i, l;
+	int ciph=CFS_BLOWFISH;
 	int cfmt=1;
 	int smsize=LARGESMSIZE;
 	unsigned char ekey[128];
 	unsigned char ek1[128];
-	unsigned int l;
 	int keycheck=1;
 	
 	while (--argc && (**++argv == '-')) {
@@ -102,8 +106,12 @@ main(argc,argv)
 			fprintf(stderr,"Can't get key\n");
 			exit(1);
 		}
-		strcpy(pword,pw);
-		if (strlen(pw)<16) {
+		l = snprintf(pword, MAX_PASSPHRASE_LEN, "%s", pw);
+		if (l < 0 || l >= MAX_PASSPHRASE_LEN) {
+			fprintf(stderr, "Key is longer than allowed %d\n", MAX_PASSPHRASE_LEN);
+			exit(1);
+		}
+		if (strnlen(pw, MAX_PASSPHRASE_LEN) < 16) {
 			fprintf(stderr,"Key must be at least 16 chars.\n");
 			exit(1);
 		}
@@ -111,25 +119,30 @@ main(argc,argv)
 			fprintf(stderr,"Can't get key\n");
 			exit(1);
 		}
-		if (strcmp(pword,pw)!=0) {
+		if (strncmp(pword, pw, MAX_PASSPHRASE_LEN) != 0) {
 		     fprintf(stderr,
 			"Keys don't match; drink some coffee and try again\n");
 		     exit(1);
 		}
 	}
 	else { /* just accept key from stdio */
-		if (fgets(pword,256,stdin) == NULL) {
+		if (fgets(pword, MAX_PASSPHRASE_LEN, stdin) == NULL) {
 			perror("cmkdir");
 			exit(1);
 		}
 		pw=pword;
-		pw[255]='\0';
-		n=strlen(pw);
+		pw[MAX_PASSPHRASE_LEN] = '\0';
+		n = strnlen(pw, MAX_PASSPHRASE_LEN);
 		if ((n>0) && (pw[n-1] == '\n'))
 			pw[n-1] = '\0';
 	}
-	if (smsize != LARGESMSIZE)
-		sprintf(pw,"%s%d",pword,smsize);
+	if (smsize != LARGESMSIZE) {
+		l = snprintf(pw, MAX_PASSPHRASE_LEN, "%s%d", pword, smsize);
+		if (l < 0 || l >= MAX_PASSPHRASE_LEN) {
+			fprintf(stderr, "For puny memory, the maximum passphrase length is about %d.\n", MAX_PASSPHRASE_LEN-1);
+			exit(1);
+		}
+	}
 	k.cipher=ciph;
 	if (cfmt==0) { 
 		if (old_pwcrunch(pw,&k)!=0) {
@@ -156,38 +169,49 @@ main(argc,argv)
 		perror("cmkdir");
 		exit(1);
 	}
-	sprintf(path,"%s/...",argv[0]);
+	l = snprintf(path, sizeof(path), "%s/...",argv[0]);
+	if (l < 0 || l >= sizeof(path)) {
+		fprintf(stderr, "Path cannot fit into NFS_MAXPATHLEN (%d); %s/...\n", NFS_MAXPATHLEN, argv[0]);
+		exit(1);
+	}
 	strcpy(str,"qua!");
 	/* now randomize the end of str.. */
 	r = trand32();
 	for (i=0; i<4; i++)
 		str[i+4]=(r<<(i*8))&0377;
 	copykey(&k,&kt);
-	cipher(&kt,str,0);
-	mask_cipher(&kt,str,1);
-	cipher(&kt,str,0);
+	cipher(&kt, (u_char *)str, 0);
+	mask_cipher(&kt, (u_char *)str, 1);
+	cipher(&kt, (u_char *)str, 0);
 	if ((fp=fopen(path,"w")) == NULL) {
 		perror("cmkdir");
 		exit(1);
 	}
 	fwrite(str,8,1,fp);
 	fclose(fp);
-	sprintf(path,"%s/..c",argv[0]);
+	sprintf(path, "%s/..c", argv[0]); /* We already know it can fit */
 	if ((fp=fopen(path,"w")) == NULL) {
 		perror("cmkdir");
 		exit(1);
 	}
 	fprintf(fp,"%d",k.cipher);
 	fclose(fp);
-	sprintf(path,"%s/..s",argv[0]);
+	sprintf(path,"%s/..s",argv[0]); /* We already know it can fit */
 	if ((fp=fopen(path,"w")) == NULL) {
 		perror("cmkdir");
 		exit(1);
 	}
 	fprintf(fp,"%d\n",smsize);
 	fclose(fp);
+	sprintf(path,"%s/..v",argv[0]); /* We already know it can fit */
+	if ((fp = fopen(path, "w")) == NULL) {
+		perror("Writing version file failed");
+		exit(1);
+	}
+	fprintf(fp, "%ld\n", datastore_version);
+	fclose(fp);
 	if (cfmt) {
-		sprintf(path,"%s/..k",argv[0]);
+		sprintf(path, "%s/..k", argv[0]); /* We know it can fit */
 		if ((fp=fopen(path,"w")) == NULL) {
 			perror("cmkdir");
 			exit(1);
