@@ -18,28 +18,31 @@
 /*
  * cfs cname - 1.3
  */
-#include <stdio.h>
-#include <rpc/rpc.h>
 #include <sys/time.h>
+
+#include <rpc/rpc.h>
+
 #include <ctype.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
+#include "cname.h"
+#include "getpass.h"
 
 /* following are never used - just so i can re-use the library */
 int validhost;
 char zerovect[]={0,0,0,0,0,0,0,0,0};
 int cursecs=0;
 
-char *gets();
-
-main(argc,argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char *argv[])
 {
 	char *pw;
-	char pword[256];
-	char *getpassword();
+	char pword[MAX_PASSPHRASE_LEN+1];
 	cfs_admkey k;
 	cfskey kt;
 	char *flg;
@@ -62,11 +65,9 @@ main(argc,argv)
 			    case '3':
 				ciph=CFS_THREE_DES;
 				break;
-#ifdef NOTDEF				
 			    case 'b':
 				ciph=CFS_BLOWFISH;
 				break;
-#endif				
 			    case 'm':
 				ciph=CFS_MACGUFFIN;
 				break;
@@ -86,7 +87,7 @@ main(argc,argv)
 		fprintf(stderr,"Can't get key\n");
 		exit(1);
 	}
-	strcpy(pword,pw);
+	strcpy(pword,pw); /* We know it's big enough */
 	k.cipher=ciph;
 	if (old_pwcrunch(pw,&k)!=0) {
 		fprintf(stderr,"Invalid key\n");
@@ -94,8 +95,8 @@ main(argc,argv)
 	}
 	copykey(&k,&kt);
 	kt.smsize=LARGESMSIZE;
-	if (((kt.primask=(char*) malloc(kt.smsize)) == NULL)
-	    || ((kt.secmask=(char*) malloc(kt.smsize)) == NULL)) {
+	if (((kt.primask=(u_char*) malloc(kt.smsize)) == NULL)
+	    || ((kt.secmask=(u_char*) malloc(kt.smsize)) == NULL)) {
 		fprintf(stderr,"No memory\n");
 		exit(2);
 	}
@@ -114,8 +115,8 @@ main(argc,argv)
 
 }
 
-int printable(s)
-     char *s;
+int
+printable(char *s)
 {
 	while (*s)
 		if (!isprint(*s++))
@@ -123,4 +124,3 @@ int printable(s)
 	return 1;
 }
 
-
