@@ -40,10 +40,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -134,9 +130,8 @@ FILE *fpmaster;
 
 #define bwrite(fp,buf,len) (fwrite(buf,len,1,fp))
 
-main(argc, argv)
-	int argc;
-	char *argv[];
+int
+main(int argc, char *argv[])
 {
 	extern char *optarg;
 	extern int optind;
@@ -241,7 +236,8 @@ main(argc, argv)
 int state=TRANS;
 int cstate=IV0;
 
-doinput()
+void
+doinput(void)
 {
 	int cc;
 	int i;
@@ -268,8 +264,8 @@ doinput()
 		done();
 }
 
-doslavein(ibuf)
-     unsigned char ibuf;
+void
+doslavein(unsigned char ibuf)
 {
 	int c;
 	static int count=0;
@@ -297,7 +293,8 @@ doslavein(ibuf)
 	}
 }
 
-slescape()
+int
+slescape(void)
 {
 	char buf;
 	int c;
@@ -334,10 +331,8 @@ slescape()
 	return 1; /* should never happen */
 }
 
-			       
-
-domasterin(ibuf)
-     unsigned char ibuf;
+void
+domasterin(unsigned char ibuf)
 {
 	int c;
 	char ch;
@@ -413,7 +408,8 @@ int pubpos=0;
 unsigned char pubbyte=0;
 int pksize = -1;
 
-startsession()
+int
+startsession(void)
 {
 	static unsigned char buf[5] = {0177, '~', 0177, '~', 'L'};
 	static unsigned char colon=':';
@@ -437,7 +433,8 @@ startsession()
 	return 0;
 }
 
-sltranspub(len)
+void
+sltranspub(int len)
 {
 	static unsigned char colon=':';
 	int i;
@@ -447,8 +444,8 @@ sltranspub(len)
 	bwrite(fpmaster,&colon,1);
 }
 
-masterkeyin(c)
-     char c;
+void
+masterkeyin(char c)
 {
 	int bits;
 
@@ -483,8 +480,8 @@ masterkeyin(c)
 	state=TRANS;
 }
 
-int mcalckeys(len)
-     int len;
+int
+mcalckeys(int len)
 {
 	int i;
 	static char kh[32];
@@ -502,8 +499,8 @@ int mcalckeys(len)
 	return 0;
 }
 
-int slcalckeys(len)
-     int len;
+int
+slcalckeys(int len)
 {
 	int i;
 	
@@ -518,8 +515,8 @@ int slcalckeys(len)
 	return 0;
 }
 
-initslkey(param)
-     int param;
+int
+initslkey(int param)
 {
 	if ((param<0) || (param>2))
 		return -1;
@@ -530,8 +527,8 @@ initslkey(param)
 	pubbyte=0;
 }
 
-slkeyin(c)
-     char c;
+void
+slkeyin(char c)
 {
 	int bits;
 
@@ -571,10 +568,8 @@ slkeyin(c)
 
 }
 
-
-
-int cipherout(ch)
-     char ch;
+int
+cipherout(char ch)
 {
 	int bits;
 	
@@ -594,8 +589,8 @@ int cipherout(ch)
 	}
 }
 
-int ciphercalcin(ch)
-     unsigned char ch;
+int
+ciphercalcin(unsigned char ch)
 {
 	static unsigned char iv[8];
 	static unsigned int cbuf;
@@ -646,7 +641,7 @@ int ciphercalcin(ch)
 #include <sys/wait.h>
 
 void
-finish()
+finish(void)
 {
 	int status;
 	register int pid;
@@ -660,7 +655,8 @@ finish()
 		done();
 }
 
-dooutput()
+void
+dooutput(void)
 {
 	register int cc;
 	int i;
@@ -687,8 +683,8 @@ dooutput()
 		done();
 }
 
-domasterout(obuf)
-     unsigned char obuf;
+void
+domasterout(unsigned char obuf)
 {
 	int c;
 	static int count=0;
@@ -710,8 +706,8 @@ domasterout(obuf)
 }
 
 
-doslaveout(obuf)
-     unsigned char obuf;
+void
+doslaveout(unsigned char obuf)
 {
 	static int bad=0;
 	int c;
@@ -794,10 +790,8 @@ doslaveout(obuf)
 	}
 }
 
-
-
-doshell(mode)
-     int mode;
+void
+doshell(int mode)
 {
 	int t;
 
@@ -825,8 +819,8 @@ doshell(mode)
 
 #ifdef NOTDEF
 /* make this work more generally */
-setprompt(mode)
-     int mode;
+void
+setprompt(int mode)
 {
 	/* TO DO - make this work for csh */
         char *oldps1;
@@ -850,9 +844,9 @@ setprompt(mode)
 }
 #endif
 
-rawtty()
+void
+rawtty(void)
 {
-
 	struct termios sbuf;
 
         sbuf = tt;
@@ -864,19 +858,22 @@ rawtty()
 	(void) tcsetattr(0, TCSAFLUSH, &sbuf);
 }
 
-cookedtty()
+void
+cookedtty(void)
 {
 	(void) tcsetattr(0, TCSAFLUSH, &tt);
 }
 
-fail()
+void
+fail(void)
 {
 
 	(void) kill(0, SIGTERM);
 	done();
 }
 
-done()
+int
+done(void)
 {
 	cookedtty();
 	switch (mode) {
@@ -896,7 +893,8 @@ done()
 
 #ifdef PTMX
 
-getmaster()
+void
+getmaster(void)
 {
 	if ((master = open("/dev/ptmx", O_RDWR)) == -1)
 		return (-1);
@@ -911,7 +909,8 @@ getmaster()
     	(void) ioctl(0, TIOCGWINSZ, (char *)&win);
 }
 
-getslave()
+void
+getslave(void)
 {
 	(void) setsid();
 	if ((slave = open(slave_name, O_RDWR)) < 0) {
@@ -932,7 +931,8 @@ getslave()
 
 #else
 
-getmaster()
+void
+getmaster(void)
 {
 	char *pty, *bank, *cp;
 	struct stat stb;
@@ -968,7 +968,8 @@ getmaster()
 	fail();
 }
 
-getslave()
+void
+getslave(void)
 {
 
 	line[strlen("/dev/")] = 't';
@@ -985,7 +986,8 @@ getslave()
 
 #endif
 
-waitenter()
+void
+waitenter(void)
 {
 	unsigned char b;
 	do {
@@ -994,9 +996,8 @@ waitenter()
 	printf("\r\n");
 }
 
-sendhex(fp,c)
-     FILE *fp;
-     unsigned int c;
+void
+sendhex(FILE *fp, unsigned int c)
 {
 	static char buf[16];
 
