@@ -12,10 +12,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the University of
- *      California, Berkeley and its contributors.
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -33,44 +29,40 @@
  * SUCH DAMAGE.
  */
 
-#include <stdio.h>
-#include <signal.h>
-#ifndef linux
-#include <sgtty.h>
-#endif
 #include <sys/types.h>
+
 #include <rpc/rpc.h>
+
+#include <fcntl.h>
+#include <signal.h>
+#include <stdio.h>
+#include <string.h>
+
 #include "nfsproto.h"
 #include "admproto.h"
 #include "cfs.h"
+#include "getpass.h"
 #include "shs.h"
 
-#if defined(irix) || defined(linux)
 /* hacks to use POSIX style termios instead of old BSD style sgttyb */
+/* Should be simply replaced with POSIX functions when possible     */
 #include <termios.h>
 #define sgttyb termios
 #define gtty(a,b) tcgetattr((a), (b))
 #define stty(a,b) tcsetattr((a), TCSAFLUSH, (b))
 #define sg_flags c_lflag
-#endif
 
 	
 char *
-getpassword(prompt)
-char *prompt;
+getpassword(char *prompt)
 {
 	struct sgttyb ttyb;
+	sig_t sig;
 	int flags;
 	register char *p;
-	register c;
+	register int c;
 	FILE *fi;
-	static char pbuf[128];
-#ifdef MACH
-	int (*signal())();
-	int (*sig)();
-#else
-	void (*sig)();
-#endif
+	static char pbuf[MAX_PASSPHRASE_LEN+1];
 
 	if ((fi = fdopen(open("/dev/tty", 2), "r")) == NULL)
 		fi = stdin;
@@ -83,8 +75,13 @@ char *prompt;
 	stty(fileno(fi), &ttyb);
 	fprintf(stderr, "%s", prompt); fflush(stderr);
 	for (p=pbuf; (c = getc(fi))!='\n' && c!=EOF;) {
-		if (p < &pbuf[127])
+		if (p < &pbuf[MAX_PASSPHRASE_LEN])
 			*p++ = c;
+		else {
+			fprintf(stderr, "\nPassphrase too long (max. %d characters)\n", MAX_PASSPHRASE_LEN);
+			pbuf[0] = '\0';
+			break;
+		}
 	}
 	*p = '\0';
 	fprintf(stderr, "\n"); fflush(stderr);
@@ -93,12 +90,11 @@ char *prompt;
 	signal(SIGINT, sig);
 	if (fi != stdin)
 		fclose(fi);
-	return(pbuf);
+	return (pbuf[0] == '\0' ? NULL : pbuf);
 }
 
-old_pwcrunch(b,k)
-     char *b;
-     cfs_admkey *k;
+int
+old_pwcrunch(char *b, cfs_admkey *k)
 {
 	int l;
 	u_char k1[8];
@@ -154,9 +150,9 @@ old_pwcrunch(b,k)
 	}
 	return 0;
 }
-new_pwcrunch(b,k)
-     char *b;
-     cfs_admkey *k;
+
+int
+new_pwcrunch(char *b, cfs_admkey *k)
 {
 	int l;
 	u_char *k1;
@@ -168,7 +164,7 @@ new_pwcrunch(b,k)
 	
 	if ((l=strlen(b))<3)
 		return -1;
-	hash = qshs(b,l);
+	hash = qshs((u_char *)b,l);
 	bcopy(hash,h1,20);
 	k1 = h1;
 	k2 = &(h1[8]);
@@ -213,9 +209,8 @@ new_pwcrunch(b,k)
 	return 0;
 }
 
-decrypt_key(k,ek)
-     cfs_admkey *k;
-     u_char *ek;
+void
+decrypt_key(cfs_admkey *k, u_char *ek)
 {
 	mcg_key mk;
 	BF_KEY bk;
@@ -263,15 +258,15 @@ decrypt_key(k,ek)
 		break;
 	    case CFS_MACGUFFIN:
 		mcg_keyset(k->cfs_admkey_u.mcgkey.primary,&mk);
-		mcg_block_decrypt(&mk,&(ek[0]));
-		mcg_block_decrypt(&mk,&(ek[8]));
+		mcg_block_decrypt(&(ek[0]), &mk);
+		mcg_block_decrypt(&(ek[8]), &mk);
 		bcopy(ek,k->cfs_admkey_u.mcgkey.primary,16);
 		bcopy(ek,k->cfs_admkey_u.mcgkey.secondary,16);
 		break;
 	    case CFS_BLOWFISH:
 		BF_set_key(&bk,16,k->cfs_admkey_u.blowkey.primary);
-		BF_encrypt(&(ek[0]),&bk,BF_DECRYPT);
-		BF_encrypt(&(ek[8]),&bk,BF_DECRYPT);
+		BF_encrypt((BF_LONG *)&(ek[0]),&bk,BF_DECRYPT);
+		BF_encrypt((BF_LONG *)&(ek[8]),&bk,BF_DECRYPT);
 		bcopy(ek,k->cfs_admkey_u.blowkey.primary,16);
 		bcopy(ek,k->cfs_admkey_u.blowkey.secondary,16);
 		break;
@@ -291,9 +286,8 @@ decrypt_key(k,ek)
 	}
 }
 
-encrypt_key(k,ek)
-     cfs_admkey *k;
-     u_char *ek;
+void
+encrypt_key(cfs_admkey *k, u_char *ek)
 {
 	mcg_key mk;
 	BF_KEY bk;
@@ -329,13 +323,13 @@ encrypt_key(k,ek)
 		break;
 	    case CFS_BLOWFISH:
 		BF_set_key(&bk,16,k->cfs_admkey_u.blowkey.primary);
-		BF_encrypt(&(ek[0]),&bk,BF_ENCRYPT);
-		BF_encrypt(&(ek[8]),&bk,BF_ENCRYPT);
+		BF_encrypt((BF_LONG *)&(ek[0]),&bk,BF_ENCRYPT);
+		BF_encrypt((BF_LONG *)&(ek[8]),&bk,BF_ENCRYPT);
 		break;
 	    case CFS_MACGUFFIN:
 		mcg_keyset(k->cfs_admkey_u.mcgkey.primary,&mk);
-		mcg_block_encrypt(&mk,&(ek[0]));
-		mcg_block_encrypt(&mk,&(ek[8]));
+		mcg_block_encrypt(&(ek[0]), &mk);
+		mcg_block_encrypt(&(ek[8]), &mk);
 		break;
 	    case CFS_SAFER_SK128:
 		Safer_Init_Module();
