@@ -1,10 +1,12 @@
+#include <inttypes.h>
+
 typedef struct {
-    long totalLength;
-    unsigned long h[5];
-    unsigned long w[80];
+    int32_t totalLength;
+    uint32_t h[5];
+    uint32_t w[80];
 } SHS_CTX;
 
-unsigned char *shs();
+unsigned char *qshs(unsigned char *s, int32_t n);
 #ifdef SOLARIS2X
 #define bzero(b, l)             memset(b, 0, l)
 #define bcopy(s, d, l)          memcpy(d, s, l)
