@@ -89,13 +89,17 @@
 #include <math.h>
 #include <stdio.h>
 
+#include <inttypes.h>
+
+#include "truerand.h"
+
 static jmp_buf env;
 static unsigned count;
 static unsigned ocount;
 static unsigned buffer;
 
-static int
-tick()
+static void
+tick(void)
 {
 	struct itimerval it;
 
@@ -107,7 +111,7 @@ tick()
 }
 
 static void
-interrupt()
+interrupt(int junk)
 {
 	if (count)
 		longjmp(env, 1);
@@ -115,8 +119,8 @@ interrupt()
 	tick();
 }
 
-static unsigned long
-roulette()
+static uint32_t
+roulette(void)
 {
 	if (setjmp(env))
 		return count;
@@ -131,15 +135,15 @@ roulette()
  * basic interface to 32 bit truerand.
  * note that any scheduled SIGALRM will be delayed by about .3 secs.
  */
-unsigned long
-raw_truerand()
+uint32_t
+raw_truerand(void)
 {
 	void (*oldalrm)();
 	struct itimerval it;
-	unsigned long counts[12];
+	uint32_t counts[12];
 	unsigned char *qshs();
 	unsigned char *r;
-	unsigned long buf;
+	uint32_t buf;
 	int i;
 
 	getitimer(ITIMER_REAL, &it);
@@ -153,14 +157,13 @@ raw_truerand()
 	setitimer(ITIMER_REAL, &it, NULL);
 
 	r = qshs(counts,sizeof(counts));
-	buf = *((unsigned long *) r);
+	buf = *((uint32_t *) r);
 
 	return buf;
 }
 
 int
-raw_n_truerand(n)
-int n;
+raw_n_truerand(int n)
 {
 	int slop, v;
 
@@ -182,8 +185,8 @@ int n;
  *	usage: 
  *		unsigned char r8;
  *		unsigned short r16;
- *		unsigned long r32;
- *		unsigned long trand8(), trand16(), trand32();
+ *		uint32_t r32;
+ *		uint32_t trand8(), trand16(), trand32();
  *		r8=trand8();
  *		r16=trand16();
  *		r32=trand32();
@@ -206,11 +209,12 @@ int n;
  * OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
  */
 
-unsigned long randbyte()
+uint32_t
+randbyte(void)
 {
-	unsigned long raw_truerand();
+	uint32_t raw_truerand();
 	unsigned char *qshs();
-	unsigned long r[2];
+	uint32_t r[2];
 	unsigned char *hash;
 
 	r[0]=raw_truerand(); r[1]=raw_truerand();
@@ -218,17 +222,20 @@ unsigned long randbyte()
 	return ((int) (*hash)) & 0xff;
 }
 
-unsigned long trand8()
+uint32_t
+trand8(void)
 {
 	return randbyte();
 }
 
-unsigned long trand16()
+uint32_t
+trand16(void)
 {
 	return randbyte() ^ (randbyte()<<8);
 }
 
-unsigned long trand32()
+uint32_t
+trand32(void)
 {
 	return randbyte() ^ (randbyte()<<8)
 		^ (randbyte()<<16) ^ (randbyte()<<24);
